library(IsingFit)
library(qgraph)

#setwd("/Users/angecramer/Documents/PAPERS/Diathesis stress major depression/NEW SIM ISING")
setwd("/Users/aojCramer/Dropbox/WORK/RESEARCH/DONE OR OTHERWISE TERMINATED/Diathesis stress major depression/PSYCH REVIEW 2/NEW SIM ISING")
read.table("KendlerData.txt",header=F)->data
data=data.matrix(data)
colnames(data)=c("dep","int","los","gai","dap","iap","iso","hso","agi","ret","fat","wor","con","dea")

#estimate Ising model on Kendler data
FittedIsing=IsingFit(data)
pdf(file="IsingModelKendlerData.pdf")
qgraph(FittedIsing$weiadj,layout="spring",filetype="pdf",filename="IsingModelKendlerData")
dev.off()

#simulation over time for three connectivity groups: stronger connections, more problems
connectivity=c(0.8,1.3,2)
pdf(file="ResultsBasicModel.pdf",width=10,height=2.5)
par(mfrow=c(1,3),col.main="white")

for (c in 1:3) {
  J=14 #number of symptoms
  N=10000 #number of timepoints
  b=abs(FittedIsing$thresholds) #threshold parameter based on Kendler data
  W=FittedIsing$weiadj #weight parameters based on Kendler data
  P=X=matrix(0,N,J) #matrix with probabilities (P) and simulated data (X)
  W=connectivity[c]*W
  for (t in 2:N) {
    A=X[t-1,]%*%W #the activation function (formula (1) in paper)
    P[t,]=1/(1+exp(b-A)) #the probability function (formula (2) in paper)
    X[t,]=1*(P[t,]>runif(J)) #symptom i becomes active (=1) if the probability is greater than randomly chosen uniformly distributed number between 0 and 1
  }
  State=apply(X,1,sum) #the totalscore of depressive symptoms per time point
  if (c==1) {StateWeak=State}
  plot(State[1:1500],type="l",xlim=c(1,1500),ylim=c(0,14),axes=F,ann=F,col="grey")
  axis(side=1,c(1,500,1000,1500))
  if (c==1) {axis(side=2,c(0,7,14))} else {axis(side=2,at=c(0,7,14),labels=F)}
}
dev.off()

#simulation over time for three connectivity groups with added stress
pdf(file="ResultsModelWithStress.pdf",width=10,height=2.5)
par(mfrow=c(1,3),col.main="white",lwd=2)
S=delta=rep(0,N)
delta[1]=1
step=0.01
Smax=15
connectivity=c(0.8,1.3,2)

for (c in 1:3) {
  J=14 #number of symptoms
  N=10000 #number of timepoints
  b=abs(FittedIsing$thresholds) #threshold parameter based on Kendler data
  W=FittedIsing$weiadj #weight parameters based on Kendler data
  P=X=matrix(0,N,J) #matrix with probabilities (P) and simulated data (X)
  W=connectivity[c]*W
  for (t in 2:N) {
    if (abs(S[t-1])>Smax) {delta[t]=-1*delta[t-1]} else {delta[t]=delta[t-1]}
    S[t]=S[t-1]+delta[t]*step
    A=X[t-1,]%*%W+S[t] #the activation function (formula (1) in paper)
    P[t,]=1/(1+exp(b-A)) #the probability function (formula (2) in paper)
    X[t,]=1*(P[t,]>runif(J)) #symptom i becomes active (=1) if the probability is greater than randomly chosen uniformly distributed number between 0 and 1
  }
  X=X[-1:-100,]
  S=S[-1:-100]
  delta=delta[-1:-100]
  
  
  S=1*S
  s=apply(X,1,sum)  
  Sseq=seq(-Smax,Smax,.2)
  Sc=cut(S,Sseq)
  test=tapply(s,list(Sc,delta),mean)
  plot(Sseq[26:100],test[c(26:100),1],type="b",col='black',axes=F,ann=F)
  lines(Sseq[26:100],test[26:100,2],type="b",col='grey',ann=F)
  
  if(c==2) mtext("Stress",side=1,line=3,col="black",adj=0.5)  
  axis(side=1,c(-10,-5,0,5))
  if (c==1) {axis(side=2,c(0,7,14))} else {axis(side=2,at=c(0,7,14),labels=F)}
  
  if(c==1) sdata=cbind(S,s) else sdata=cbind(sdata,s)
  if(c==1) mdata=cbind(Sseq[-1],tapply(s,list(Sc,delta),mean)) else mdata=cbind(mdata,tapply(s,list(Sc,delta),mean))
  
}
dev.off()
